#ifndef __ENODE_HH__
#define __ENODE_HH__

#include <vector>
#include <map>
#include "tp.hh"
#include "sp.hh"
#include "pp.hh"
#include "nodeattr.hh"
#include "sched.hh"
#include "node.hh"

//Common: FIXME (see opts.hh, and snode.hh)
typedef std::map< pvertex_id, vertex_id > node_map_t;

class ENode : public Node {

public:
  ENode(NodeAttr *, TimeProfile *, Architecture *);
  ENode(const ENode &);
  ~ENode();

  std::pair< bool, float >
  insert(const vertex_set_t & _chi, vertex_id & _root, DAG & _dag, node_map_t & _arg_map, 
	 pattern_t & _pattern, TargetInstruction *_target_instruction, 
	 Architecture * _architecture, int _offset);
  Schedule *get_schedule(void);
  NodeAttr *get_node_attr(void) const;
  float get_e(void); //Energy dissipation
  int get_tau(void) const;
  ENode & operator=(const ENode &);
  bool operator==(const ENode &) const;

  std::pair< bool, std::set< int > >
  get_residence_class(vertex_id _v) { 
    if(space_profile->find(_v) == space_profile->end()) {
      return make_pair(false, std::set<int>());
    }
    return make_pair(true, (*space_profile)[_v]);
  }
  SpaceProfile *get_space_profile(void) { return space_profile; }
  void set_node_attr(NodeAttr *_node_attr) { node_attr = _node_attr; }
  size_t get_hash(void) const { return 0; }

private:
  TimeProfile *time_profile;
  SpaceProfile *space_profile;
  PowerProfile *power_profile;
  Schedule *s;
  NodeAttr *node_attr;
  float e; //accumulated dissipated energy
  int reference_time;

};

#endif //__ENODE_HH__
